@@ -46,6 +46,8 @@ has _data => (
 		baselabel
 		linear
 		wordsep
+		direction
+		change_direction
 		start
 		end
 		cached_table
@@ -195,12 +197,15 @@ The readings may be specified by object or ID.
 
 Returns all Relationship objects in the collation.
 
-=head2 add_relationship( $reading, $other_reading, $options )
+=head2 add_relationship( $reading, $other_reading, $options, $changed_readings )
 
 Adds a new relationship of the type given in $options between the two readings,
 which may be specified by object or ID.  Returns a value of ( $status, @vectors)
 where $status is true on success, and @vectors is a list of relationship edges
-that were ultimately added.
+that were ultimately added. If an array reference is passed in as $changed_readings, 
+then any readings that were altered due to the relationship creation are added to
+the array.
+
 See L<Text::Tradition::Collation::Relationship> for the available options.
 
 =cut 
@@ -210,7 +215,7 @@ sub BUILDARGS {
 	my %args = @args == 1 ? %{ $args[0] } : @args;
 	# TODO determine these from the Moose::Meta object
 	my @delegate_attrs = qw(sequence relations readings wit_list_separator baselabel 
-		linear wordsep start end cached_table _graphcalc_done);
+		linear wordsep direction start end cached_table _graphcalc_done);
 	my %data_args;
 	for my $attr (@delegate_attrs) {
 		$data_args{$attr} = delete $args{$attr} if exists $args{$attr};
@@ -314,6 +319,7 @@ The first two arguments may be either readings or reading IDs.
 
 =begin testing
 
+use Test::More::UTF8;
 use Text::Tradition;
 use TryCatch;
 
@@ -367,6 +373,25 @@ try {
 	ok( 0, "Caught a rank exception: " . $e->message );
 }
 
+# Test right-to-left reading merge.
+my $rtl = Text::Tradition->new( 
+    'name'  => 'inline', 
+    'input' => 'Tabular',
+    'sep_char' => ',',
+    'direction' => 'RL',
+    'file'  => 't/data/arabic_snippet.csv'
+    );
+my $rtlc = $rtl->collation;
+is( $rtlc->reading('r8.1')->text, 'سبب', "Got target first reading in RTL text" );
+my $pt = $rtlc->path_text('A');
+my @path = $rtlc->reading_sequence( $rtlc->start, $rtlc->end, 'A' );
+is( $rtlc->reading('r9.1')->text, 'صلاح', "Got target second reading in RTL text" );
+$rtlc->merge_readings( 'r8.1', 'r9.1', 1 );
+is( $rtlc->reading('r8.1')->text, 'سبب صلاح', "Got target merged reading in RTL text" );
+is( $rtlc->path_text('A'), $pt, "Path text is still correct" );
+is( scalar($rtlc->reading_sequence( $rtlc->start, $rtlc->end, 'A' )), 
+	scalar(@path) - 1, "Path was shortened" );
+
 =end testing
 
 =cut
@@ -744,8 +769,11 @@ sub duplicate_reading {
 				$args{$attr->name} = $which eq 'Array' 
 					? [ $r->$acc ] : { $r->$acc };
 			} 
-		} else {
-			$args{$attr->name} = $r->$acc if $acc;
+		} elsif( $acc ) {
+			my $attrval = $r->$acc;
+			if( defined $attrval ) {
+				$args{$attr->name} = $attrval;
+			}
 		}
 	}
 	# By definition the new reading will no longer be common.
@@ -899,15 +927,27 @@ sub clear_witness {
 
 sub add_relationship {
 	my $self = shift;
-    my( $source, $target, $opts ) = $self->_stringify_args( @_ );
+    my( $source, $target, $opts, $altered_readings ) = $self->_stringify_args( @_ );
     my( @vectors ) = $self->relations->add_relationship( $source, $target, $opts );
+    my $did_recalc;
     foreach my $v ( @vectors ) {
-    	next unless $self->get_relationship( $v )->colocated;
-    	if( $self->reading( $v->[0] )->has_rank && $self->reading( $v->[1] )->has_rank
-    		&& $self->reading( $v->[0] )->rank ne $self->reading( $v->[1] )->rank ) {
+    	my $rel = $self->get_relationship( $v );
+    	next unless $rel->colocated;
+    	my $r1 = $self->reading( $v->[0] );
+    	my $r2 =  $self->reading( $v->[1] );
+    	# If it's a spelling or orthographic relationship, and one is marked
+    	# as a lemma, set the normal form on the non-lemma to reflect that.
+    	if( $r1->does( 'Text::Tradition::Morphology' ) ) {
+    		my @changed = $r1->relationship_added( $r2, $rel );
+    		if( ref( $altered_readings ) eq 'ARRAY' ) {
+    			push( @$altered_readings, @changed );
+    		}
+    	}
+    	next if $did_recalc;
+    	if( $r1->has_rank && $r2->has_rank && $r1->rank ne $r2->rank ) {
     			$self->_graphcalc_done(0);
     			$self->_clear_cache;
-    			last;
+    			$did_recalc = 1;
     	}
     }
     return @vectors;
@@ -956,6 +996,118 @@ sub reading_witnesses {
 Returns an SVG string that represents the graph, via as_dot and graphviz.
 See as_dot for a list of options.  Must have GraphViz (dot) installed to run.
 
+=begin testing
+
+use File::Which;
+use Text::Tradition;
+use XML::LibXML;
+use XML::LibXML::XPathContext;
+
+
+SKIP: {
+	skip( 'Need Graphviz installed to test graphs', 16 )
+		unless File::Which::which( 'dot' );
+
+	my $datafile = 't/data/Collatex-16.xml';
+
+	my $tradition = Text::Tradition->new( 
+		'name'  => 'inline', 
+		'input' => 'CollateX',
+		'file'  => $datafile,
+		);
+	my $collation = $tradition->collation;
+
+	# Test the svg creation
+	my $parser = XML::LibXML->new();
+	$parser->load_ext_dtd( 0 );
+	my $svg = $parser->parse_string( $collation->as_svg() );
+	is( $svg->documentElement->nodeName(), 'svg', 'Got an svg document' );
+
+	# Test for the correct number of nodes in the SVG
+	my $svg_xpc = XML::LibXML::XPathContext->new( $svg->documentElement() );
+	$svg_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	my @svg_nodes = $svg_xpc->findnodes( '//svg:g[@class="node"]' );
+	is( scalar @svg_nodes, 26, "Correct number of nodes in the graph" );
+
+	# Test for the correct number of edges
+	my @svg_edges = $svg_xpc->findnodes( '//svg:g[@class="edge"]' );
+	is( scalar @svg_edges, 32, "Correct number of edges in the graph" );
+
+	# Test svg creation for a subgraph
+	my $part_svg = $parser->parse_string( $collation->as_svg( { from => 15 } ) ); # start, no end
+	is( $part_svg->documentElement->nodeName(), 'svg', "Got an svg subgraph to end" );
+	my $part_xpc = XML::LibXML::XPathContext->new( $part_svg->documentElement() );
+	$part_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	@svg_nodes = $part_xpc->findnodes( '//svg:g[@class="node"]' );
+	is( scalar( @svg_nodes ), 9, 
+		"Correct number of nodes in the subgraph" );
+	@svg_edges = $part_xpc->findnodes( '//svg:g[@class="edge"]' );
+	is( scalar( @svg_edges ), 10,
+		"Correct number of edges in the subgraph" );
+
+	$part_svg = $parser->parse_string( $collation->as_svg( { from => 10, to => 13 } ) ); # start, no end
+	is( $part_svg->documentElement->nodeName(), 'svg', "Got an svg subgraph in the middle" );
+	$part_xpc = XML::LibXML::XPathContext->new( $part_svg->documentElement() );
+	$part_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	@svg_nodes = $part_xpc->findnodes( '//svg:g[@class="node"]' );
+	is( scalar( @svg_nodes ), 9, 
+		"Correct number of nodes in the subgraph" );
+	@svg_edges = $part_xpc->findnodes( '//svg:g[@class="edge"]' );
+	is( scalar( @svg_edges ), 11,
+		"Correct number of edges in the subgraph" );
+
+
+	$part_svg = $parser->parse_string( $collation->as_svg( { to => 5 } ) ); # start, no end
+	is( $part_svg->documentElement->nodeName(), 'svg', "Got an svg subgraph from start" );
+	$part_xpc = XML::LibXML::XPathContext->new( $part_svg->documentElement() );
+	$part_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	@svg_nodes = $part_xpc->findnodes( '//svg:g[@class="node"]' );
+	is( scalar( @svg_nodes ), 7, 
+		"Correct number of nodes in the subgraph" );
+	@svg_edges = $part_xpc->findnodes( '//svg:g[@class="edge"]' );
+	is( scalar( @svg_edges ), 7,
+		"Correct number of edges in the subgraph" );
+
+	# Test a right-to-left graph
+	my $arabic = Text::Tradition->new(
+		input => 'Tabular',
+		sep_char => ',',
+		name => 'arabic',
+		direction => 'RL',
+		file => 't/data/arabic_snippet.csv' );
+	my $rl_svg = $parser->parse_string( $arabic->collation->as_svg() );
+	is( $rl_svg->documentElement->nodeName(), 'svg', "Got an svg subgraph from start" );
+	my $rl_xpc = XML::LibXML::XPathContext->new( $rl_svg->documentElement() );
+	$rl_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	my %node_cx;
+	foreach my $node ( $rl_xpc->findnodes( '//svg:g[@class="node"]' ) ) {
+		my $nid = $node->getAttribute('id');
+		$node_cx{$nid} = $rl_xpc->findvalue( './svg:ellipse/@cx', $node );
+	}
+	my @sorted = sort { $node_cx{$a} <=> $node_cx{$b} } keys( %node_cx );
+	is( $sorted[0], '__END__', "End node is the leftmost" );
+	is( $sorted[$#sorted], '__START__', "Start node is the rightmost" );
+	
+	# Now try making it bidirectional
+	$arabic->collation->change_direction('BI');
+	my $bi_svg = $parser->parse_string( $arabic->collation->as_svg() );
+	is( $bi_svg->documentElement->nodeName(), 'svg', "Got an svg subgraph from start" );
+	my $bi_xpc = XML::LibXML::XPathContext->new( $bi_svg->documentElement() );
+	$bi_xpc->registerNs( 'svg', 'http://www.w3.org/2000/svg' );
+	my %node_cy;
+	foreach my $node ( $bi_xpc->findnodes( '//svg:g[@class="node"]' ) ) {
+		my $nid = $node->getAttribute('id');
+		$node_cy{$nid} = $rl_xpc->findvalue( './svg:ellipse/@cy', $node );
+	}
+	@sorted = sort { $node_cy{$a} <=> $node_cy{$b} } keys( %node_cy );
+	is( $sorted[0], '__START__', "Start node is the topmost" );
+	is( $sorted[$#sorted], '__END__', "End node is the bottom-most" );
+	
+
+} #SKIP
+
+=end testing
+
 =cut
 
 sub as_svg {
@@ -1020,9 +1172,11 @@ sub as_dot {
     $graph_name = join( '_', split( /\s+/, $graph_name ) );
 
     my %graph_attrs = (
-    	'rankdir' => 'LR',
     	'bgcolor' => 'none',
     	);
+    unless( $self->direction eq 'BI' ) {
+    	$graph_attrs{rankdir} = $self->direction;
+    }
     my %node_attrs = (
     	'fontsize' => 14,
     	'fillcolor' => 'white',
@@ -1994,7 +2148,7 @@ sub readings_at_rank {
 		$readings{$e->{'t'}->id} = $e->{'t'};
 	}
 	return values %readings;
-}		
+}
 
 =head2 next_reading( $reading, $sigil );
 
@@ -2105,28 +2259,44 @@ sub common_readings {
 	return @common;
 }
 
-=head2 path_text( $sigil, [, $start, $end ] )
+=head2 path_text( $sigil [, $start, $end, $use_normal_form ] )
 
-Returns the text of a witness (plus its backup, if we are using a layer)
-as stored in the collation.  The text is returned as a string, where the
+Returns the text of a witness (plus its backup, if we are using a layer) as
+stored in the collation.  The text is returned as a string, where the
 individual readings are joined with spaces and the meta-readings (e.g.
-lacunae) are omitted.  Optional specification of $start and $end allows
-the generation of a subset of the witness text.
+lacunae) are omitted.  Optional specification of $start and $end allows the
+generation of a subset of the witness text. Optional specification of
+$use_normal_form produces a text based on the normal form, rather than the
+raw text, of the reading.
 
 =cut
 
 sub path_text {
-	my( $self, $wit, $start, $end ) = @_;
+	my( $self, $wit, $start, $end, $normal ) = @_;
 	$start = $self->start unless $start;
 	$end = $self->end unless $end;
 	my @path = grep { !$_->is_meta } $self->reading_sequence( $start, $end, $wit );
+	return $self->known_path_text( $normal, @path );
+}
+
+=head2 known_path_text( $use_normal_form, @sequence ) 
+
+Returns the text of a given sequence of readings. No attempt is made to
+validate the sequence in question. If $use_normal_form is set to true, the
+normal form of each reading in the sequence will be used to construct the
+text.
+
+=cut
+
+sub known_path_text {
+	my( $self, $normal, @path ) = @_;
 	my $pathtext = '';
 	my $last;
 	foreach my $r ( @path ) {
 		unless ( $r->join_prior || !$last || $last->join_next ) {
 			$pathtext .= ' ';
 		} 
-		$pathtext .= $r->text;
+		$pathtext .= $normal ? $r->normal_form : $r->text;
 		$last = $r;
 	}
 	return $pathtext;
