@@ -1,6 +1,7 @@
 package Text::Tradition::Collation::Data;
 use Moose;
 use Graph;
+use Text::Tradition::Datatypes;
 
 has 'sequence' => (
     is => 'ro',
@@ -69,6 +70,13 @@ has 'wordsep' => (
 	isa => 'Str',
 	default => ' ',
 	);
+	
+has 'direction' => (
+	is => 'ro',
+	isa => 'TextDirection', 
+	default => 'LR',
+	writer => 'change_direction',
+	);
     
 has 'start' => (
 	is => 'ro',
