@@ -1,11 +1,7 @@
 package Text::Tradition::Collation::Relationship;
 
 use Moose;
-use Moose::Util::TypeConstraints;
-
-enum 'RelationshipScope' => qw( local document global );
-
-no Moose::Util::TypeConstraints;
+use Text::Tradition::Datatypes;
 
 =head1 NAME
 
@@ -49,12 +45,17 @@ or across all traditions.
 the text to have different meanings. Possible values are 0 (no), 1 (slightly),
 and >1 (yes).
 
-=item * non_correctable - (Optional) True if the reading would not have been 
-corrected independently.
+=item * a_derivable_from_b - (Optional) True if the first reading is likely to 
+
+=item * b_derivable_from_a - (Optional) True if the second reading is likely to
 
 =item * non_independent - (Optional) True if the variant is unlikely to have 
 occurred independently in unrelated witnesses.
 
+=item * is_significant - (Optional) Indicates whether, in the opinion of the scholar,
+the variation in question is stemmatically significant. Possible values are 'yes',
+'maybe', and 'no'.
+
 =back
 
 =head1 ACCESSORS
@@ -67,10 +68,14 @@ occurred independently in unrelated witnesses.
 
 =head2 annotation
 
-=head2 non_correctable
+=head2 a_derivable_from_b
+
+=head2 b_derivable_from_a
 
 =head2 non_independent
 
+=head2 is_significant
+
 See the option descriptions above.
 
 =cut
@@ -117,16 +122,27 @@ has 'alters_meaning' => (
 	default => 0,
 	);
 
-has 'non_correctable' => (
+has 'a_derivable_from_b' => (
 	is => 'ro',
 	isa => 'Bool',
 	);
-
+	
+has 'b_derivable_from_a' => (
+	is => 'ro',
+	isa => 'Bool',
+	);
+	
 has 'non_independent' => (
 	is => 'ro',
 	isa => 'Bool',
 	);
 	
+has 'is_significant' => (
+	is => 'ro',
+	isa => 'Ternary',
+	default => 'no',
+	);
+	
 around 'alters_meaning' => sub {
 	my $orig = shift;
 	my $self = shift;
