@@ -3,6 +3,7 @@ package Text::Tradition::Parser::CollateX;
 use strict;
 use warnings;
 use Text::Tradition::Parser::GraphML qw/ graphml_parse /;
+use TryCatch;
 
 =head1 NAME
 
@@ -49,6 +50,7 @@ binmode STDOUT, ":utf8";
 binmode STDERR, ":utf8";
 eval { no warnings; binmode $DB::OUT, ":utf8"; };
 
+# Test a simple CollateX input
 my $cxfile = 't/data/Collatex-16.xml';
 my $t = Text::Tradition->new( 
     'name'  => 'inline', 
@@ -69,6 +71,32 @@ if( $t ) {
     is( $related[0]->id, 'n18', "Correct transposition link" );
 }
 
+# Now test a CollateX result with a.c. witnesses
+
+my $ct = Text::Tradition->new( 
+	name => 'florilegium',
+	input => 'CollateX',
+	file => 't/data/florilegium_cx.xml' );
+
+is( ref( $ct ), 'Text::Tradition', "Parsed the CollateX input" );
+if( $ct ) {
+    is( scalar $ct->collation->readings, 309, "Collation has all readings" );
+    is( scalar $ct->collation->paths, 361, "Collation has all paths" );
+    is( scalar $ct->witnesses, 13, "Collation has correct number of witnesses" );
+    
+    my %layered = ( E => 1, P => 1, Q => 1, T => 1 );
+    foreach my $w ( $ct->witnesses ) {
+    	is( $w->is_layered, $layered{$w->sigil}, 
+    		"Witness " . $w->sigil . " has correct layered setting" );
+    }
+    
+    my $pseq = $ct->witness('P')->text;
+    my $pseqac = $ct->witness('P')->layertext;
+    is( scalar @$pseq, 264, "Witness P has correct number of tokens" );
+    is( scalar @$pseqac, 261, "Witness P (a.c.) has correct number of tokens" );
+}
+    
+
 =end testing
 
 =cut
@@ -114,6 +142,7 @@ sub parse {
     }
         
     # Now add the path edges.
+    my %transpositions;
     foreach my $e ( @{$graph_data->{'edges'}} ) {
         my $from = $e->{'source'};
         my $to = $e->{'target'};
@@ -125,30 +154,94 @@ sub parse {
 			## Add the path for each witness listesd.
             # Create the witness objects if they does not yet exist.
             foreach my $wit ( split( /, /, $e->{$WITKEY} ) ) {
-				unless( $tradition->witness( $wit ) ) {
-					$tradition->add_witness( 
-						'sigil' => $wit, 'sourcetype' => 'collation' );
+            	my $sigil = _base_sigil( $collation, $wit ) || $wit;
+            	my $wit_object = $tradition->witness( $sigil );
+				unless( $wit_object ) {
+					$wit_object = $tradition->add_witness( 
+						sigil => $sigil, 
+						sourcetype => 'collation' );
+				}
+				$wit_object->is_collated(1);
+				if( $wit ne $sigil ) {
+					$wit_object->is_layered(1);
 				}
 				$collation->add_path( $from->{$IDKEY}, $to->{$IDKEY}, $wit );
 			}
-        } else { # type 'relationship'
-        	if( $collation->linear ) {
-				$collation->add_relationship( $from->{$IDKEY}, $to->{$IDKEY},
-					{ 'type' => 'transposition' } );
-			} else {
-				$collation->merge_readings( $from->{$IDKEY}, $to->{$IDKEY} );
-			}
+        } else { # CollateX-marked transpositions
+			# Save the transposition links so that we can apply them 
+			# once they are all collected.
+			$transpositions{ $from->{$IDKEY} } = $to->{$IDKEY};
         }
     }
-
-    # Rank the readings.
-    $collation->calculate_common_readings()
-    	if $collation->linear; # will implicitly rank
+    
+    # Mark initialization as done so that relationship validation turns on
+    $tradition->_init_done( 1 );
+    # Now apply transpositions as appropriate.
+    if( $collation->linear ) {
+    	# Sort the transpositions by reading length, then try first to merge them
+    	# and then to transpose them. Warn if the text isn't identical.
+    	foreach my $k ( sort { 
+				my $t1 = $collation->reading( $a )->text;
+				my $t2 = $collation->reading( $b )->text;
+				return length( $t2 ) <=> length( $t1 );
+    		} keys %transpositions ) {
+    		my $v = $transpositions{$k};
+    		my $merged;
+			try {
+				$collation->add_relationship( $k, $v, { type => 'collated' } );
+				$merged = 1;
+			} catch ( Text::Tradition::Error $e ) {
+				1;
+			}
+    		unless( $merged ) {
+    			my $transpopts = { type => 'transposition' };
+    			unless( $collation->reading( $k )->text eq $collation->reading( $v )->text ) {
+    				$transpopts->{annotation} = 'CollateX fuzzy match';
+    			}
+				try {
+					$collation->add_relationship( $k, $v, $transpopts );
+				} catch ( Text::Tradition::Error $e ) {
+					warn "Could neither merge nor transpose $k and $v; DROPPING transposition";
+				}
+	    	}		
+    	}
+    
+    	# Rank the readings and find the commonalities
+    	unless( $opts->{'nocalc'} ) {
+			$collation->calculate_ranks();
+			$collation->flatten_ranks();
+			$collation->calculate_common_readings();
+		}
+    } else {
+    	my %merged;
+    	foreach my $k ( keys %transpositions ) {
+    		my $v = $transpositions{$k};
+    		$k = $merged{$k} if exists $merged{$k};
+    		$v = $merged{$v} if exists $merged{$v};
+    		next if $k eq $v;
+    		if( $collation->reading( $k )->text eq $collation->reading( $v )->text ) {
+    			$collation->merge_readings( $k, $v );
+    			$merged{$v} = $k;
+    		} else {
+    			warn "DROPPING transposition link for non-identical readings $k and $v";
+    		}
+    	}
+    }
 
     # Save the text for each witness so that we can ensure consistency
     # later on
 	$tradition->collation->text_from_paths();	
 }
+
+sub _base_sigil {
+	my( $collation, $sigil ) = @_;
+	my $acstr = $collation->ac_label;
+	if( $sigil =~ /^(.*)\Q$acstr\E$/ ) {
+		return $1;
+	}
+	return undef;
+}
+	
     
 =head1 BUGS / TODO
 
