@@ -73,19 +73,21 @@ is( ref( $t ), 'Text::Tradition', "Parsed a JSON alignment" );
 if( $t ) {
     is( scalar $t->collation->readings, 26, "Collation has all readings" );
     is( scalar $t->collation->paths, 32, "Collation has all paths" );
-    is( scalar $t->witnesses, 3, "Collation has all witnesses" );
+    is( scalar $t->witnesses, 2, "Collation has all witnesses" );
 }
 
 my %seen_wits;
-map { $seen_wits{$_} = 0 } qw/ A B C /;
+map { $seen_wits{$_} = 0 } qw/ A C /;
 # Check that we have the right witnesses
 foreach my $wit ( $t->witnesses ) {
 	$seen_wits{$wit->sigil} = 1;
 }
-is( scalar keys %seen_wits, 3, "No extra witnesses were made" );
+is( scalar keys %seen_wits, 2, "No extra witnesses were made" );
 foreach my $k ( keys %seen_wits ) {
 	ok( $seen_wits{$k}, "Witness $k still exists" );
 }
+# Check that witness A is layered
+ok( $t->witness('A')->is_layered, "Witness A has its pre-correction layer" );
 
 # Check that the witnesses have the right texts
 foreach my $wit ( $t->witnesses ) {
@@ -94,6 +96,16 @@ foreach my $wit ( $t->witnesses ) {
 	is( $graphtext, $origtext, "Collation matches original for witness " . $wit->sigil );
 }
 
+# Check that the ranks are right
+is( $t->collation->end->rank, 19, "Ending node has the correct rank" );
+foreach my $rdg ( $t->collation->readings ) {
+	next if $rdg->is_meta;
+	my $idrank = $rdg->id;
+	$idrank =~ s/^r(\d+)\..*$/$1/;
+	is( $idrank, $rdg->rank, "Reading $rdg has the correct rank" );
+}
+
+
 =end testing
 
 =cut
@@ -108,6 +120,15 @@ sub parse {
 	my @witnesses; # Keep the ordered list of our witnesses
     my %ac_wits;  # Track these for later removal
     foreach my $sigil ( map { $_->{'witness'} } @{$table->{'alignment'}} ) {
+    	# Get the appropriate sigil.
+        my $aclabel = $c->ac_label;
+        if( $sigil =~ /^(.*)\Q$aclabel\E$/ ) {
+        	my $real_sig = $1;
+        	my $layer_sig = $real_sig . '_ac';
+            $ac_wits{$sigil} = { layer => $layer_sig, base => $real_sig };
+            # use XML Name version of this, since ' (a.c.)' will break XML validation
+        	$sigil = $layer_sig;
+        }
     	my $wit;
     	if( $tradition->has_witness( $sigil ) ) {
     		$wit = $tradition->witness( $sigil );
@@ -118,10 +139,6 @@ sub parse {
 		}
         $wit->path( [ $c->start ] );
         push( @witnesses, $wit );
-        my $aclabel = $c->ac_label;
-        if( $sigil =~ /^(.*)\Q$aclabel\E$/ ) {
-            $ac_wits{$sigil} = $1;
-        }
     }
     
     # Save the original witness text for consistency checking. We do this
@@ -131,7 +148,7 @@ sub parse {
     foreach my $intext ( @{$table->{'alignment'}} ) {
     	my $rs = $intext->{'witness'};
     	my $is_layer = exists $ac_wits{$rs};
-    	my $wit = $tradition->witness( $is_layer ? $ac_wits{$rs} : $rs );
+    	my $wit = $tradition->witness( $is_layer ? $ac_wits{$rs}->{base} : $rs );
     	my @tokens = grep { $_ && $_->{'t'} !~ /^\#.*\#$/ } @{$intext->{'tokens'}};
     	my @words = map { _restore_punct( $_ ) } @tokens;
     	$is_layer ? $wit->layertext( \@words ) : $wit->text( \@words );
@@ -154,10 +171,10 @@ sub parse {
             } # else skip it for empty readings.
         }
     }
-    
+
     # Collapse our lacunae into a single node and
     # push the end node onto all paths.
-    $c->end->rank( $length );
+    $c->end->rank( $length+1 );
     foreach my $wit ( @witnesses ) {
         my $p = $wit->path;
         my $last_rdg = shift @$p;
@@ -176,8 +193,8 @@ sub parse {
     # Fold any a.c. witnesses into their main witness objects, and
     # delete the independent a.c. versions.
     foreach my $a ( keys %ac_wits ) {
-    	my $ac_wit = $tradition->witness( $a );
-        my $main_wit = $tradition->witness( $ac_wits{$a} );
+    	my $ac_wit = $tradition->witness( $ac_wits{$a}->{layer} );
+        my $main_wit = $tradition->witness( $ac_wits{$a}->{base} );
         next unless $main_wit;
         $main_wit->uncorrected_path( $ac_wit->path );
         $tradition->del_witness( $ac_wit );
@@ -213,8 +230,8 @@ sub make_nodes {
 			if( exists( $unique{$word} ) ) {
 				$rdg = $unique{$word};
 			} else {
-				my %args = ( 'id' => 'r' . join( '.', $idx, $j+1 ),
-					'rank' => $idx,
+				my %args = ( 'id' => 'r' . join( '.', $idx+1, $j+1 ),
+					'rank' => $idx+1,
 					'text' => $word,
 					'collation' => $c );
 				if( $word eq '#LACUNA#' ) {
